﻿using AutoMapper;
using BizService;
using Microsoft.Extensions.Logging;
using MiniExcelLibs;
using Model.Dto;
using Model.Entities;
using PLCTool.Common;
using Prism.Commands;
using Prism.Mvvm;
using Prism.Services.Dialogs;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;

namespace PLCTool.ViewModels.BusinessManageViewModel
{
    

    public class DeviceTestViewModel : BindableBase
    {
        private readonly IBasicDeviceService _iBasicDeviceService;
        private readonly IBasicDeviceKindService _iBasicDeviceKindService;
        private readonly IBasicProjectService _iBasicProjectService;
        private readonly IBasicPlcTestSchemeService _basicPlcTestSchemeService;
        private readonly IMapper _mapper;
        private readonly IDialogService _dialog;
        private readonly ILogger _logger;
        private List<BasDeviceWithSchModel> allDeviceList = new List<BasDeviceWithSchModel>();//所有方案
        private List<BasDeviceWithSchModel> conditionDevices = new List<BasDeviceWithSchModel>();//符合条件的方案
        public DeviceTestViewModel(IBasicDeviceService iBasicDeviceService, IBasicDeviceKindService iBasicDeviceKindService, IBasicProjectService iBasicProjectService, IBasicPlcTestSchemeService basicPlcTestSchemeService, IMapper mapper, IDialogService dialog, ILogger logger)
        {
            _iBasicDeviceService = iBasicDeviceService;
            _iBasicDeviceKindService = iBasicDeviceKindService;
            _iBasicProjectService = iBasicProjectService;
            _basicPlcTestSchemeService = basicPlcTestSchemeService;
            _mapper = mapper;
            _dialog = dialog;
            _logger = logger;
            QueryCommand = new DelegateCommand<object>(Query);
          
            ExportCommand = new DelegateCommand<string>(Export);
            ResetCommand = new DelegateCommand<object>(Reset);
            OnLoadCommand = new DelegateCommand(OnLoad);
            //手动  自动测试
            AutoTestCommand = new DelegateCommand<object>(AutoDest);
            ManualTestCommand = new DelegateCommand<object>(ManualTest);
         
        }








        #region 私有方法
        /// <summary>
        /// 加载页面
        /// </summary>
        private void OnLoad()
        {
            DeviceKindNameList.Clear();
            DeviceKindNameList.Add("---");
            ProjectNameList.Clear();
            ProjectNameList.Add("---");
            var Kinds = _iBasicDeviceKindService.FindAllDeviceKind();
            foreach (var kind in Kinds)
            {
                DeviceKindNameList.Add(kind);
            }
            var projects = _iBasicProjectService.FindAllProject();
            foreach (var project in projects)
            {
                ProjectNameList.Add(project);
            }
            DeviceKindName = "---";
            ProjectName = "---";

            GetProjectConfig();
        }
        private void Reset(object obj)
        {
            DeviceNo = string.Empty;
            DeviceName = string.Empty;
            DeviceKindName = "---";
            ProjectName = "---";


        }

        /// <summary>
        /// 自动测试
        /// </summary>
        /// <param name="obj"></param>
        private void AutoDest(object obj)
        {
            object[] multiObj = obj as object[];
           
            //测试方案明细主键ID
            long id =(long) multiObj[0];
            long deviceId= (long)multiObj[1];
            DialogParameters parm = new DialogParameters();
            parm.Add("Key", id);
            parm.Add("Key2", deviceId);
            //弹出详情对话框
            //弹出详情对话框
            _dialog.ShowDialog("AutoTestView", parm, async callback =>
            {
                if (callback.Result == ButtonResult.OK)
                {
                    //更新表格，重新获取
            
                }

            });
        }

        private void ManualTest(object obj)
        {
            object[] multiObj = obj as object[];

            //测试方案明细主键ID
            long id = (long)multiObj[0];
            long deviceId = (long)multiObj[1];
            DialogParameters parm = new DialogParameters();
            parm.Add("Key", id);
            parm.Add("Key2", deviceId);
            //弹出详情对话框
            //弹出详情对话框
            _dialog.ShowDialog("ManualTestView", parm, async callback =>
            {
                if (callback.Result == ButtonResult.OK)
                {
                    //更新表格，重新获取
                   
                }

            });
        }

        /// <summary>
        /// 查询
        /// </summary>
        /// <param name="obj"></param>
        private void Query(object obj)
        {
            conditionDevices = (from a in allDeviceList
                                where (string.IsNullOrEmpty(DeviceName) ? true : (a.DeviceName == DeviceName))
                                && (string.IsNullOrEmpty(DeviceNo) ? true : (a.DeviceNo == DeviceNo))
                                && (DeviceKindName == "---") ? true : (a.DeviceKindName == DeviceKindName)
                                   && (ProjectName == "---") ? true : (a.ProjectName == ProjectName)
                                select a).ToList();
            //默认显示的第一页
            conditionDevices = conditionDevices.OrderBy(x => x.DeviceId).ToList();
            Getpage();
        }

        private void Export(string obj)
        {
            using (System.Windows.Forms.SaveFileDialog saveFileDialog = new System.Windows.Forms.SaveFileDialog()
            {
                //设置文件类型
                //书写规则例如：txt files(*.txt)|*.txt
                Filter = "Excel files(*.xlsx)|*.xlsx|All files(*.*)|*.*",
                //设置默认文件名（可以不设置）
                FileName = "项目配置表",

                //获取或设置一个值，该值指示如果用户省略扩展名，文件对话框是否自动在文件名中添加扩展名。（可以不设置）
                AddExtension = true,

                //保存对话框是否记忆上次打开的目录
                RestoreDirectory = true
            })
            {
                // Show save file dialog box

                //点了保存按钮进入
                if (saveFileDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    try
                    {
                        //获得文件路径
                        string localFilePath = saveFileDialog.FileName.ToString();

                        //获取文件内容
                        MiniExcel.SaveAs(localFilePath, DeviceItemList);
                    }
                    catch (Exception ex)
                    {
                        _logger.LogError(ex.ToString());
                    }


                }

            }
        }

        /// <summary>
        /// 获取所有项目
        /// </summary>
        private void GetProjectConfig()
        {
            allDeviceList.Clear();
            conditionDevices.Clear();
            var devicelist = _iBasicDeviceService.QueryList();
            var allDeviceKinds = _mapper.Map<List<bas_device>, List<BasDeviceDto>>(devicelist);
            //所有测试方案
            var schlist = _basicPlcTestSchemeService.QueryList();
            var schDtoList = _mapper.Map<List<bas_plc_test_scheme>, List<BasicPlcTestSchemeDto>>(schlist);
            

            foreach (var item in allDeviceKinds)
            {
                string deviceKind =_iBasicDeviceKindService.Find((int)item.DeviceKindId)?.devicekind_name;
                //在测试方案中查找此设备类型的所有方案
                var schs= schDtoList.FindAll(x => x.DeviceKindName == deviceKind);
                //设备方案
                foreach (var sch in schs)
                {
                    allDeviceList.Add(new BasDeviceWithSchModel()
                    {
                        DeviceId = item.DeviceId,
                        DeviceNo = item.DeviceNo,
                        DeviceName = item.DeviceName,
                        DeviceKindName = _iBasicDeviceKindService.Find((int)item.DeviceKindId)?.devicekind_name,
                        ProjectName = _iBasicProjectService.Find((int)item.ProjectId)?.project_name,
                        SchemeName = sch.SchemeName,
                        SchemeId=sch.SchemeId,
                    }) ;
                    conditionDevices.Add(new BasDeviceWithSchModel()
                    {
                        DeviceId = item.DeviceId,
                        DeviceNo = item.DeviceNo,
                        DeviceName = item.DeviceName,
                        DeviceKindName = _iBasicDeviceKindService.Find((int)item.DeviceKindId)?.devicekind_name,
                        ProjectName = _iBasicProjectService.Find((int)item.ProjectId)?.project_name,
                        SchemeName = sch.SchemeName,
                        SchemeId = sch.SchemeId,
                    });
                }
             
            }
            conditionDevices = conditionDevices.OrderBy(x => x.DeviceId).ToList();
            Getpage();
        }
        /// <summary>
        /// 获取页面
        /// </summary>
        private void Getpage()
        {
            CurrentPage = 1;
            TotalCount = conditionDevices.Count;
            CurrentPageChanged();


        }
        /// <summary>
        /// 页面变化
        /// </summary>
        private void CurrentPageChanged()
        {

            DeviceItemList.Clear();

            foreach (var i in conditionDevices.Skip((CurrentPage - 1) * CountPerPage).Take(CountPerPage))
            {
                DeviceItemList.Add(i);
            }
        }

        #endregion
        #region 命令绑定

        public DelegateCommand<object> QueryCommand { set; get; }

        public DelegateCommand<string> ExportCommand { set; get; }

  
        public DelegateCommand<object> ResetCommand { set; get; }

        public DelegateCommand OnLoadCommand { set; get; }

        public DelegateCommand<object> AutoTestCommand { set; get; }

        public DelegateCommand<object> ManualTestCommand { set; get; }
        #endregion

        #region 数据绑定
        private ObservableCollection<BasDeviceWithSchModel> deviceItemList = new ObservableCollection<BasDeviceWithSchModel>();
        public ObservableCollection<BasDeviceWithSchModel> DeviceItemList
        {
            get { return deviceItemList; }
            set { deviceItemList = value; RaisePropertyChanged(); }
        }

        /// <summary>
        /// 设备编号
        /// </summary>
        private string devicedNo;
        public string DeviceNo
        {
            get { return devicedNo; }
            set { devicedNo = value; RaisePropertyChanged(); }
        }

        /// <summary>
        /// 设备类型名称
        /// </summary>
        private string deviceName;
        public string DeviceName
        {
            get { return deviceName; }
            set { deviceName = value; RaisePropertyChanged(); }
        }

        private string projectName;
        public string ProjectName
        {
            get { return projectName; }
            set { projectName = value; RaisePropertyChanged(); }
        }
        private string deviceKindName;
        public string DeviceKindName
        {
            get { return deviceKindName; }
            set { deviceKindName = value; RaisePropertyChanged(); }
        }

        /// <summary>
        /// 项目名称
        /// </summary>
        private ObservableCollection<string> projectNameList = new ObservableCollection<string>();
        public ObservableCollection<string> ProjectNameList
        {
            get { return projectNameList; }
            set { projectNameList = value; RaisePropertyChanged(); }
        }
        /// <summary>
        /// 设备类型
        /// </summary>
        private ObservableCollection<string> deviceKindNameList = new ObservableCollection<string>();
        public ObservableCollection<string> DeviceKindNameList
        {
            get { return deviceKindNameList; }
            set { deviceKindNameList = value; RaisePropertyChanged(); }
        }
        /// <summary>
        /// 总条数
        /// </summary>
        private int totalCount;
        public int TotalCount
        {
            get { return totalCount; }
            set { totalCount = value; RaisePropertyChanged(); CurrentPageChanged(); }
        }
        /// <summary>
        /// 每页数量
        /// </summary>
        private int countPerPage = 15;
        public int CountPerPage
        {
            get { return countPerPage; }
            set { countPerPage = value; RaisePropertyChanged(); CurrentPageChanged(); }
        }
        /// <summary>
        /// 单前页
        /// </summary>
        private int currentPage = 1;
        public int CurrentPage
        {
            get { return currentPage; }
            set { currentPage = value; RaisePropertyChanged(); CurrentPageChanged(); }
        }

        #endregion
    }
}
